// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:equatable/equatable.dart';
import 'package:wyatt_authentication_bloc/src/models/user/user_interface.dart';
import 'package:wyatt_authentication_bloc/src/repositories/authentication_repository_interface.dart';

part 'authentication_state.dart';

class AuthenticationCubit extends Cubit<AuthenticationState> {
  final AuthenticationRepositoryInterface _authenticationRepository;

  StreamSubscription<UserInterface>? _userSubscription;

  final Future<Map<String, dynamic>> Function(UserInterface user)?
      _onAuthSuccess;

  AuthenticationCubit({
    required AuthenticationRepositoryInterface authenticationRepository,
    Future<Map<String, dynamic>> Function(UserInterface user)? onAuthSuccess,
  })  : _authenticationRepository = authenticationRepository,
        _onAuthSuccess = onAuthSuccess,
        super(const AuthenticationState.unknown());

  Future<void> init() async {
    final _firstUser = await _authenticationRepository.user.first;
    start();
    return changeStatus(_firstUser);
  }

  bool start() {
    _userSubscription = _authenticationRepository.user.listen(changeStatus);
    return true;
  }

  bool stop() {
    _userSubscription?.cancel();
    return true;
  }

  Future<void> changeStatus(UserInterface user) async {
    if (user.isNotEmpty) {
      final Map<String, dynamic>? userData = await _onAuthSuccess?.call(user);
      emit(AuthenticationState.authenticated(user, userData));
    } else {
      stop();
      emit(const AuthenticationState.unauthenticated());
    }
  }

  void logOut() {
    unawaited(_authenticationRepository.signOut());
  }

  @override
  Future<void> close() {
    _userSubscription?.cancel();
    return super.close();
  }
}
