// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/widgets.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

abstract class BlocBaseConsumerScreen<B extends BlocBase<S>, S extends Object>
    extends StatelessWidget {
  const BlocBaseConsumerScreen({super.key});

  /// Takes the previous `state` and the current `state` and is responsible for
  /// returning a [bool] which determines whether or not to trigger
  /// [onBuild] with the current `state`.
  bool shouldBuildWhen(S previous, S current) => true;

  /// Takes the previous `state` and the current `state` and is responsible for
  /// returning a [bool] which determines whether or not to trigger
  /// [onListen] with the current `state`.
  bool shouldListenWhen(S previous, S current) => true;

  /// The [onBuild] function which will be invoked on each widget build.
  /// The [onBuild] takes the `BuildContext` and current `state` and
  /// must return a widget.
  Widget onBuild(BuildContext context, S state);

  /// Takes the `BuildContext` along with the `state`
  /// and is responsible for executing in response to `state` changes.
  void onListen(BuildContext context, S state) {}

  @override
  Widget build(BuildContext context) {
    return BlocConsumer<B, S>(
      listenWhen: shouldListenWhen,
      listener: onListen,
      buildWhen: shouldBuildWhen,
      builder: onBuild,
    );
  }
}
