// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:meta/meta.dart';
import 'package:wyatt_form_bloc/src/form/form.dart';

@immutable
class FormEntry {
  const FormEntry(this.key, this.input, {this.export = true, String? name})
      : _field = name ?? key;

  final String key;
  final FormInput input;
  final bool export;
  final String _field;

  String get name => _field;

  FormEntry copyWith({
    String? key,
    FormInput? input,
    bool? export,
    String? name,
  }) {
    return FormEntry(
      key ?? this.key,
      input ?? this.input,
      export: export ?? this.export,
      name: name,
    );
  }

  FormEntry clone() {
    return FormEntry(
      key,
      input,
      export: export,
      name: name,
    );
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is FormEntry &&
        other.key == key &&
        other.input == input &&
        other.export == export &&
        other.name == name;
  }

  @override
  int get hashCode {
    return key.hashCode ^
        input.hashCode ^
        export.hashCode ^
        name.hashCode;
  }

  @override
  String toString() {
    return 'FormEntry(key: $key, input: $input, '
        'export: $export, name: $name)';
  }
}
